#!/bin/bash
#
# Copyright (C) 2020 Actiphy, Inc.
#   All rights reserved.
#
# THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF Actiphy, Inc.
# The copyright notice above does not evidence any actual or intended
# publication of such source code.
#
# AIP-packages-tool.sh: AIP Packages installation tool.
# Created by Yoshinobu Ito
# File:  AIP-packages-tool.sh
#
# Date:  10/19/20
#
# Hist:
#	ver 0.1.11
#
# Qualified Platforms:
#	Redhat 6.x, 7.x .8.x
#	CentOS 6.x, 7.x, 8.x
#
# Requirements:
#	AIP Linux version 2018 Update5 or later.
#
export LANG=C
ScVer="0.1.11"

FIFO=/tmp/$$.fifo
LOG=/tmp/AIP-packages-tool.log
mkfifo ${FIFO}
trap '/bin/rm -f ${FIFO}' exit
tee ${LOG} < ${FIFO} &
exec > ${FIFO} 2>&1

dir=$(dirname $(readlink -f $0))

if [ `id -u` -ne 0 ]; then
	echo -e "${color_red} Only root user can install this product.${color_off}"
fi

color_off="\e[m"
color_red="\e[31m"
color_blue="\e[34m"
color_magenta="\e[35m"
color_blue_back="\e[44m"

#xxd -r -p ./kernel-release > kernel-release.txt

req_wget=`rpm -qa | grep -w wget | wc -l`
req_bc=`rpm -qa | grep -w bc | wc -l`

if [ $req_wget -eq 0 -o $req_bc -eq 0 ]; then
	yum install -y -q wget bc
	if [ $? -ne 0 ]; then
		echo -e "${color_red}This script requires wget and bc command.${color_off}"
		echo -e "${color_red}Please install wget and bc rpm.${color_off}"
		exit
	fi
fi

print_head()
{
	clear
	echo -e "${color_blue_back}******************************************************************${color_off}"
	echo -e "${color_blue_back}** ActiveImage Protector 2018 Linux Packages installation tool  **${color_off}"
	echo -e "${color_blue_back}******************************************************************${color_off}"
	echo ""
}

print_head
echo "** `date '+%Y-%m-%d %H:%M:%S'` -Ver $ScVer START"

change_xsession(){

if [ ${RHELVER} -eq 8 ]; then
	xs_bk="/usr/share/xsessions/bk_aip"
	ws_bk="/usr/share/wayland-sessions/bk_aip"

	chk_gdm=`systemctl list-unit-files | grep gdm.service | wc -l`
	if [ $chk_gdm -eq 1 ]; then
		if [ ! -d $xs_bk -o ! -d $ws_bk ]; then
			echo -e "${color_blue}Chenge the login session to classic.${color_off}"
			echo -e "${color_blue}Please logout after this process.${color_off}"
			cd /usr/share/xsessions
			mkdir bk_aip
			mv *.desktop bk_aip
			mv bk_aip/gnome-classic.desktop ./

			cd /usr/share/wayland-sessions
			mkdir bk_aip
			mv *.desktop bk_aip
			echo -e "${color_red}Now restart gdm.service.${color_off}"
			echo -e "${color_red}Caution: Be careful because force logout.${color_off}"
			read -p "=== Press any key to continue...  To cancel the command (ctrl + c)."
			systemctl restart gdm.service
		fi
	fi
fi
}

oracle_repo() {
	echo [ol_latest] >> /etc/yum.repos.d/ol_temp.repo
	echo name=ol_latest >> /etc/yum.repos.d/ol_temp.repo
	if [ ${RHELVER} -eq 6 ]; then
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL6/latest/x86_64/ >> /etc/yum.repos.d/ol_temp.repo
	elif [ ${RHELVER} -eq 7 ]; then
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL7/latest/x86_64/ >> /etc/yum.repos.d/ol_temp.repo
	elif [ ${RHELVER} -eq 8 ]; then
		echo baseurl=https://yum.oracle.com/repo/OracleLinux/OL8/baseos/latest/x86_64/ >> /etc/yum.repos.d/ol_temp.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/ol_temp.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/ol_temp.repo

	chk_ol=`rpm -qa | grep oraclelinux-release | wc -l`
	if [ $chk_ol -eq 0 ]; then
		if [ ${RHELVER} -eq 6 ]; then
			echo "Checking yum repository ....."
			wget -q https://yum.oracle.com/RPM-GPG-KEY-oracle-ol6 -O /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			gpg --quiet --with-fingerprint /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			yum -y -q install oraclelinux-release-el6
		elif [ ${RHELVER} -eq 7 ]; then
			echo "Checking yum repository ....."
			wget -q https://yum.oracle.com/RPM-GPG-KEY-oracle-ol7 -O /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			gpg --quiet --with-fingerprint /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			yum -y -q install oraclelinux-release-el7
		elif [ ${RHELVER} -eq 8 ]; then
			echo "Checking yum repository ....."
 			wget -q https://yum.oracle.com/RPM-GPG-KEY-oracle-ol8 -O /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
              	gpg -q --import /etc/pki/rpm-gpg/RPM-GPG-KEY-oracle
			yum -y -q install oraclelinux-release-el8
		fi
	fi
}

vault_repo_base() {
#no update
	echo [vault_base] >> /etc/yum.repos.d/vault.repo
	echo name=vault_base >> /etc/yum.repos.d/vault.repo
	if [ ${RHELVER} -eq 7 ]; then
		echo baseurl=http://vault.centos.org/$release_ver1/os/$plat/ >> /etc/yum.repos.d/vault.repo
	elif [ ${RHELVER} -eq 8 ]; then 
		echo baseurl=http://vault.centos.org/$release_ver1/BaseOS/$plat/os/ >> /etc/yum.repos.d/vault.repo
	else 
		echo baseurl=http://vault.centos.org/$os_ver/os/$plat/ >> /etc/yum.repos.d/vault.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/vault.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/vault.repo
}

vault_repo_base2() {
#major update
	echo [vault_base] >> /etc/yum.repos.d/vault.repo
	echo name=vault_base >> /etc/yum.repos.d/vault.repo
	if [ ${RHELVER} -eq 7 ]; then
		echo baseurl=http://vault.centos.org/$release_ver1/os/$plat/ >> /etc/yum.repos.d/vault.repo
	elif [ ${RHELVER} -eq 8 ]; then 
		echo baseurl=http://vault.centos.org/$release_ver1/BaseOS/$plat/os/ >> /etc/yum.repos.d/vault.repo
	else 
		echo baseurl=http://vault.centos.org/$os_ver/os/$plat/ >> /etc/yum.repos.d/vault.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/vault.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/vault.repo
}

vault_repo_update() {

	echo [vault_updates] >> /etc/yum.repos.d/vault.repo
	echo name=vault_updates >> /etc/yum.repos.d/vault.repo
	if [ ${RHELVER} -eq 7 ]; then
		echo baseurl=http://vault.centos.org/$release_ver1/updates/$plat/ >> /etc/yum.repos.d/vault.repo
	elif [ ${RHELVER} -eq 8 ]; then 
		echo baseurl=http://vault.centos.org/$release_ver1/BaseOS/$plat/os/ >> /etc/yum.repos.d/vault.repo
	else
		echo baseurl=http://vault.centos.org/$release_ver/updates/$plat/ >> /etc/yum.repos.d/vault.repo
	fi
	echo enable=1 >> /etc/yum.repos.d/vault.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/vault.repo
}

vault_remove() {

	if [ -f /etc/yum.repos.d/vault.repo ]; then
		rm -rf /etc/yum.repos.d/vault.repo
	fi
}

DVD_remove() {
	if [ -f /etc/yum.repos.d/DVD.repo ]; then
		rm -rf /etc/yum.repos.d/DVD*.repo
	fi
}

ol_remove() {
	if [ -f /etc/yum.repos.d/ol_temp.repo ]; then
		rm -rf /etc/yum.repos.d/ol_temp.repo
		rm -rf /etc/yum.repos.d/oracle*.repo
		rm -rf /etc/yum.repos.d/uek*.repo
		rm -rf /etc/yum.repos.d/virt-ol*.repo
	fi
}

yum_proxy() {

	export http_proxy="http://$IPadd_name:$port"
	export proxy_username="$user"
	export proxy_password="$pass"
}

repo() {
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/ >> /etc/yum.repos.d/DVD.repo
	echo enable=1 >> /etc/yum.repos.d/DVD.repo
}

repo_ent5() {
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/Server >> /etc/yum.repos.d/DVD.repo
	echo enable=1 >> /etc/yum.repos.d/DVD.repo
}

ker_dev_not_inst_match() {
	
	echo -e "${color_blue}kernel-devel is not installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is match.${color_off}"
	uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
	if [ $? -eq 0 ]; then
		echo "Minor update"
		vault_repo_update
		echo "$MISSING1 install"
		yum -y --disablerepo=* --enablerepo=vault_updates --nogpgcheck install $MISSING1
			if [ $? -eq 1 ]; then
				echo -e "${color_red}yum install is failed in vault repository.${color_off}"
				echo -e "${color_red}Try another repository.${color_off}"
			fi
		ins_check_devel2=`rpm -qa kernel-devel | wc -l`
		if [ $ins_check_devel2 = 0 ]; then
		vault_remove
		#yum clean all
		yum -y install $MISSING1
			if [ $? -eq 1 ]; then
				echo -e "${color_red}yum install is failed.${color_off}"
				echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
				echo -e "${color_red}Please install "$MISSING1".${color_off}"
				echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
				exit 1
			fi
		fi
	else
		echo "Base"
		vault_repo_base
		echo "$MISSING1 install"
		yum -y --disablerepo=* --enablerepo=vault_base --nogpgcheck install $MISSING1
			if [ $? -eq 1 ]; then
				echo -e "${color_red}yum install is failed in vault repository.${color_off}"
				echo -e "${color_red}Try another repository.${color_off}"
			fi
		ins_check_devel2=`rpm -qa kernel-devel | wc -l`
		if [ $ins_check_devel2 = 0 ]; then
		vault_remove
		#yum clean all
		yum -y install $MISSING1
			if [ $? -eq 1 ]; then
				echo -e "${color_red}yum install is failed.${color_off}"
				echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
				echo -e "${color_red}Please install "$MISSING1".${color_off}"
				echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
				exit 1
			fi
		fi
	fi
}

ker_dev_not_inst_not_match() {

	echo -e "${color_blue}kernel-devel is not installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is not match.${color_off}"
	echo "Major update"		
	echo "Base"
	vault_repo_base2
	echo "$MISSING1 install"
	yum -y --disablerepo=* --enablerepo=vault_base --nogpgcheck install $MISSING1
		if [ $? -eq 1 ]; then
			echo -e "${color_red}yum install is failed in vault repository.${color_off}"
			echo -e "${color_red}Try another repository.${color_off}"
		fi
	ins_check_devel2=`rpm -qa kernel-devel | wc -l`
	if [ $ins_check_devel2 = 0 ]; then
	vault_remove
	#yum clean all
	yum -y install $MISSING1
		if [ $? -eq 1 ]; then
			echo -e "${color_red}yum install is failed.${color_off}"
			echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
			echo -e "${color_red}Please install "$MISSING1".${color_off}"
			echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
			exit 1
		fi
	fi
}

ker_dev_inst_match() {

	echo -e "${color_blue}kernel-devel is installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is match.${color_off}"
	ker_ver=`uname -r`
	ker_dev_ver=`rpm -qa kernel-devel | grep -oE '[0-9]+.[0-9]+.[0-9]+.-[0-9]+.*'`
	echo "Checking kernel and kernel-devel...."
	if [ $ker_ver = $ker_dev_ver ]; then
		echo "kernel and kernel-devel is match."
	else 
		echo "kernel and kernel-devel is not match."
		echo "Checking kernel update...."
		uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
		if [ $? -eq 0 ]; then
			echo "Minor update"
			vault_repo_update
			echo "$MISSING1 install"
			yum -y --disablerepo=* --enablerepo=vault_updates --nogpgcheck install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed in vault repository.${color_off}"
					echo -e "${color_red}Try another repository.${color_off}"
				fi
			ins_check_devel2=`rpm -qa kernel-devel | wc -l`
			if [ $ins_check_devel2 = 0 ]; then
			vault_remove
			#yum clean all
			yum -y install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed.${color_off}"
					echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
					echo -e "${color_red}Please install "$MISSING1".${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					exit 1
				fi
			fi
		else
			echo "Base"
			vault_repo_base
			echo "$MISSING1 install"
			yum -y --disablerepo=* --enablerepo=vault_base --nogpgcheck install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed in vault repository.${color_off}"
					echo -e "${color_red}Try another repository.${color_off}"
				fi
			ins_check_devel2=`rpm -qa kernel-devel | wc -l`
			if [ $ins_check_devel2 = 0 ]; then
			vault_remove
			#yum clean all
			yum -y install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed.${color_off}"
					echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
					echo -e "${color_red}Please install "$MISSING1".${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					exit 1
				fi
			fi
		fi
	fi
}

ker_dev_inst_not_match() {

	echo -e "${color_blue}kernel-devel is installed.${color_off}"
	echo -e "${color_blue}OS Ver and kernel ver is not match.${color_off}"
	ker_ver=`uname -r`
	ker_dev_ver=`rpm -qa kernel-devel | grep -oE '[0-9]+.[0-9]+.[0-9]+.-[0-9]+.*'`
	echo "Checking kernel and kernel-devel...."
	if [ $ker_ver = $ker_dev_ver ]; then
		echo "kernel and kernel-devel is match."
	else 
		echo "kernel and kernel-devel is not match."
		echo "Checking kernel update...."
		uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
		if [ $? -eq 0 ]; then
			echo "Major update"
			vault_repo_update
			echo "$MISSING1 install"
			yum -y --disablerepo=* --enablerepo=vault_updates --nogpgcheck install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed in vault repository.${color_off}"
					echo -e "${color_red}Try another repository.${color_off}"
				fi
			ins_check_devel2=`rpm -qa kernel-devel | wc -l`
			if [ $ins_check_devel2 = 0 ]; then
			vault_remove
			#yum clean all
			yum -y install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed.${color_off}"
					echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
					echo -e "${color_red}Please install "$MISSING1".${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					exit 1
				fi
			fi
		else
			echo "Base"
			vault_repo_base2
			echo "$MISSING1 install"
			yum -y --disablerepo=* --enablerepo=vault_base --nogpgcheck install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed in vault repository.${color_off}"
					echo -e "${color_red}Try another repository.${color_off}"
				fi
			ins_check_devel2=`rpm -qa kernel-devel | wc -l`
			if [ $ins_check_devel2 = 0 ]; then
			vault_remove
			#yum clean all
			yum -y install $MISSING1
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed.${color_off}"
					echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
					echo -e "${color_red}Please install "$MISSING1".${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					exit 1
				fi
			fi
		fi
	fi

}

OS_Ver()
{

if [ -f /etc/redhat-release ]; then
	RHELVER=`sed -e 's/^.*release \([0-9]\).*$/\1/' /etc/redhat-release`
fi

}

#Detect RedHat or CentOS
Check_Dist()
{
	if [ -f /etc/redhat-release ]; then
		OSDIST=`cat /etc/redhat-release | grep CentOS | wc -l`
		#OSDIST=0 ---RHEL	
		#OSDIST=1 ---CentOS
	fi
}

# Support OS check
# 5.5- 64bit redhat,CentOS
Check_OS_Version()
{
   if [ "$RHELVER" == "5" ] ; then
      #V2=`cat /etc/redhat-release | grep -oE '[0-9]+\.[0-9]+'`
       #   if [ "$V2" == "" -o "$V2" == "5.1" -o "$V2" == "5.2" -o "$V2" == "5.3" -o "$V2" == "5.4" ] ; then
             echo " This OS version does not support."
             exit 1
       #   fi
   fi
	if [ ! -f /etc/redhat-release ]; then
		echo " This Distribution does not support."
		exit 1
	fi
	bit_chk=`arch | grep -o x86_64`
	if [ $? = 1 ]; then
		echo " 32bit OS does not support."
		exit 1
	fi
}

Check_sb()	#Check SecureBoot
{
if [ $RHELVER -ge 7 ]; then
	sb=`mokutil --sb-state 2>/dev/null | grep "SecureBoot enabled" | wc -l`
	if [ $sb -eq 1 ]; then
		echo "This system is enabled SecureBoot."
		echo "AIP Linux dose not support Secureboot."
		echo "Please disable SecureBoot and try install."
		exit 1
	fi
fi
}

Check_GLIB()
{
	GLIBCXX=`strings /usr/lib64/libstdc++.so.6 | grep "GLIBCXX_3.4.15" | wc -l`
	GLIBC=`strings /lib64/libc.so.6 | grep "GLIBC_2.7" | wc -l`
	if [ $GLIBCXX -eq 0 -o $GLIBC -eq 0 ]; then
		echo -e "${color_red}lease update [gcc glibc] packages.${color_off}"
	fi
}

Check_internet()
{
	wget --spider -q -nv --timeout 10 -t 1 http://www.google.com
	if [ $? -eq 0 ]; then
		net_ok=0	#connect ok
	else
		net_ok=1	#not connect
	fi
}

#RHEL Only Function
Check_subsc()
{
	Check_Dist
	if [ $OSDIST -eq 0 ]; then
		if [ "$RHELVER" == "5" ]; then
			subsc_stat=`subscription-manager list | grep Status: | awk '{print $2,$3}'`
			if [ "$subsc_stat" = "Not Subscribed" ]; then
				subsc=1	#subscription unknown
			else
				subsc=0	#subscription ok
			fi
		else
			subsc_stat=`subscription-manager list --matches="Red Hat*" | grep Status: | awk '{print $2}'`
			if [ $subsc_stat = "Unknown" ]; then
				subsc=1	#subscription unknown
			else
				subsc=0	#subscription ok
			fi
		fi
		echo "Subscription Status: "$subsc_stat
	fi
}

repo() 
{
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/ >> /etc/yum.repos.d/DVD.repo
	echo enabled=1 >> /etc/yum.repos.d/DVD.repo
}

repo8() 
{
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/BaseOS/ >> /etc/yum.repos.d/DVD.repo
	echo enabled=1 >> /etc/yum.repos.d/DVD.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/DVD.repo
	echo ""
	echo [DVD1] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD1 >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/AppStream/ >> /etc/yum.repos.d/DVD.repo
	echo enabled=1 >> /etc/yum.repos.d/DVD.repo
	echo gpgcheck=0 >> /etc/yum.repos.d/DVD.repo
}

repo_ent5() 
{
	echo [DVD] >> /etc/yum.repos.d/DVD.repo
	echo name=DVD >> /etc/yum.repos.d/DVD.repo
	echo baseurl=file://$dvd_path/Server >> /etc/yum.repos.d/DVD.repo
	echo enabled=1 >> /etc/yum.repos.d/DVD.repo
}

Check_DVD_path()
{
	if [ ! -e "$dvd_path" ]; then
		echo "[$dvd_path]" "Path not found."
		exit
	fi
}

Cent5_net() 
{
	echo "=== Internet connection OK."
	if [ "$subsc" == "0" ]; then
		echo "=== echo Subscription Status: $subsc_stat."
	fi
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}" 			
	yum update $EXIST
	yum install $MISSING
	Check_GLIB
}

Cent5_nonet() 
{	
	if [ "$net_ok" == "0" ]; then
		echo "=== Internet connection OK."
	else
		echo "=== Internet connection failed. Please insert the OS DVD."
	fi
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo_ent5)
	echo -e "${color_blue}Install the required module using yum command.${color_off}"
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck update $EXIST
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install $MISSING
	Check_GLIB
}

Cent6_later_net() 
{
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}" 	
	if [ $Agent -ne 1 ]; then
		if [ $RHELVER -le 7 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ "$?" = 1 ]; then	
				yum -y install redhat-lsb.x86_64	
			else
				yum -y upgrade redhat-lsb.x86_64	
			fi
		fi
		if [ $RHELVER -ge 8 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ "$?" = 1 ]; then	
				dnf -y install redhat-lsb.x86_64	
			else
				dnf -y upgrade redhat-lsb.x86_64	
			fi
		fi
	fi
	if [ $RHELVER -le 7 ]; then
		yum -y update $EXIST
		yum -y install $MISSING
	fi
	if [ $RHELVER -ge 8 ]; then
		dnf -y update $EXIST
		dnf -y install $MISSING
	fi
}

Cent6_7_nonet() 
{
	echo "=== Internet connection failed. Please insert the OS DVD."
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo)
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}"
	if [ $Agent -ne 1 ]; then
		yum list installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install redhat-lsb.x86_64
		else
			yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck update $EXIST
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install $MISSING
}

Cent8_nonet() 
{
	echo "=== Internet connection failed. Please insert the OS DVD."
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo8)
	echo -e "${color_blue}Install the required rpm packages using dnf command.${color_off}"
	if [ $Agent -ne 1 ]; then
		dnf list --installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install redhat-lsb.x86_64
		else
			dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck update $EXIST
	dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $MISSING
}

RHEL5_net() 
{
	echo "=== Internet connection OK."
	if [ "$subsc" == "0" ]; then
		echo "=== echo Subscription Status: $subsc_stat."
	fi
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}" 			
	yum -y update $EXIST
	yum -y install $MISSING
	Check_GLIB
}

RHEL5_nonet() 
{	
	if [ "$net_ok" == "0" ]; then
		echo "=== Internet connection OK."
	else
		echo "=== Internet connection failed. Please insert the OS DVD."
	fi
	if [ "$subsc" == "1" ]; then
		echo "=== Subscription Status: $subsc_stat. Please insert the OS DVD."
	else
		echo "=== Subscription Status: $subsc_stat."
	fi
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo_ent5)
	echo -e "${color_blue}Install the required module using yum command.${color_off}"
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck update $EXIST
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install $MISSING
	Check_GLIB
}

RHEL6_later_net() 
{
	echo "=== Internet connection OK."
	if [ "$subsc" == "0" ]; then
		echo "=== echo Subscription Status: $subsc_stat."
	fi
	echo -e "${color_blue}Install the required rpm packages using yum command.${color_off}" 			
	if [ $Agent -ne 1 ]; then
		if [ $RHELVER -le 7 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ "$?" = 1 ]; then	
				yum -y install redhat-lsb.x86_64	
			else
				yum -y upgrade redhat-lsb.x86_64			
			fi
		fi
		if [ $RHELVER -ge 8 ]; then
			yum list installed | grep redhat-lsb >> /dev/null
			if [ "$?" = 1 ]; then	
				dnf -y install redhat-lsb.x86_64			
			else
				dnf -y upgrade redhat-lsb.x86_64			
			fi
		fi
	fi
	if [ $RHELVER -le 7 ]; then
		yum -y update $EXIST
		yum -y install $MISSING
	fi
	if [ $RHELVER -ge 8 ]; then
		dnf -y update $EXIST
		dnf -y install $MISSING
	fi
}

RHEL6_7_nonet() 
{	
	if [ "$net_ok" == "0" ]; then
		echo "=== Internet connection OK."
	else
		echo "=== Internet connection failed. Please insert the OS DVD."
	fi
	if [ "$subsc" == "1" ]; then
		echo "=== Subscription Status: $subsc_stat. Please insert the OS DVD."
	else
		echo "=== Subscription Status: $subsc_stat."
	fi
	echo -n "Input DVD mount path [e.g.: run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	Check_DVD_path
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo)
	echo -e "${color_blue}Install the required module using yum command.${color_off}"
	if [ $Agent -ne 1 ]; then
		yum list installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install redhat-lsb.x86_64
		else
			yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck update $EXIST
	yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install $MISSING
}

RHEL8_nonet() 
{
	if [ "$net_ok" == "0" ]; then
		echo "=== Internet connection OK."
	else
		echo "=== Internet connection failed. Please insert the OS DVD."
	fi
	if [ "$subsc" == "1" ]; then
		echo "=== Subscription Status: $subsc_stat. Please insert the OS DVD."
	else
		echo "=== Subscription Status: $subsc_stat."
	fi
	echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
	read dvd_path
	if [ ! -e $dvd_path ]; then
		echo "Path not found."
	fi
	dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
	$(repo8)
	echo -e "${color_blue}Install the required module using yum command.${color_off}"
	if [ $Agent -ne 1 ]; then
		dnf list --installed | grep redhat-lsb >> /dev/null
		if [ "$?" = 1 ]; then	
			dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install redhat-lsb.x86_64
		else
			dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck upgrade redhat-lsb.x86_64
		fi
	fi
	dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck update $EXIST
	dnf -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $MISSING
}

number_select()
{
   local title="$1"
   echo ""
   
   local i=0
   local j=1
   while [ "$#" -gt "1" ] ; do
      (( i=i+1 ))
      printf "[$i] $2\n"
      shift
   done

   echo ""
   while [ 1 ] ; do
	  printf "${title} "
	  if [ $i -eq 1 ] ; then
	     printf "[1]:"
	  else
	     printf "[1-${i}]:"
	  fi
	  read op
	  for ((j=1; j<=$i; j=j+1)) ; do
	     [[ "$op" == "$j" ]] && return $op
      done
   done
}

select_options()
{
	while [ 1 ] ; do
		number_select "Please select install menu from" "Agent" "GUI" "[1]-[2]All" "Kernel-devel" "Install datto driver" "Uninstall datto driver" "help" "exit"
	  	case "$?" in
			1) Agent ;;
			2) GUI ;;
			3) All ;;
			4) pre_kernel-devel
				kernel_devel_install ;;
			5) pre_kernel-devel
				datto_install ;;
			6) pre_kernel-devel
				datto_remove ;;
			7) show_help ;;
			8) break ;;
		esac
	done
}

select_options_5()
{
	while [ 1 ] ; do
		number_select "Please select install component for Agent" "Agent" "Exit"
	  	case "$?" in
			1) Agent ;;
			2) break ;;
		esac
	done
}

redhat()
{
	if [ "$net_ok" == "0" ]; then
		if [ "$subsc" == "0" ]; then 
			#net ok and subscription ok
			case "$RHELVER" in
			5)
				RHEL5_net
				;;
			6)
				RHEL6_later_net
				;;
			7)
				RHEL6_later_net
				;;
			8)
				RHEL6_later_net
				;;
			esac
		else
			#net ok and subscription x
			case "$RHELVER" in
			5)
				RHEL5_nonet
				;;
			6)
				RHEL6_7_nonet
				;;
			7)
				RHEL6_7_nonet
				;;
			8)
				RHEL8_nonet
				;;
			esac
		fi
	else
		#net x and subscription x
		case "$RHELVER" in
		5)
			RHEL5_nonet
			;;
		6)
			RHEL6_7_nonet
			;;
		7)
			RHEL6_7_nonet
			;;
		8)
			RHEL8_nonet
			;;
		esac
	fi
}

CentOS()
{
	if [ $net_ok -eq 0 ]; then 
		#net ok and subscription ok
		case "$RHELVER" in
		5)
			Cent5_nonet
			;;
		6)
			Cent6_later_net
			;;
		7)
			Cent6_later_net
			;;
		8)
			Cent6_later_net
			;;
		esac
	else
		#net ok and subscription x
		case "$RHELVER" in
		5)
			Cent5_nonet
			;;
		6)
			Cent6_7_nonet
			;;
		7)
			Cent6_7_nonet
			;;
		8)
			Cent8_nonet
			;;
		esac
fi
}

Agent()
{	
	print_head

	echo -e "${color_blue}[ Install Agent rpm ]${color_off}"
	echo ""

	unset MISSING
	unset EXIST

	Agent=1
	for RPM_e in ${List_Agent}; do
		if rpm -q $RPM_e &>/dev/null; then
			if [[ $RPM_e = *.x86_64 ]]; then
					EXIST="$RPM_e $EXIST"
			fi
		fi
	done

	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for RPM in ${List_Agent}; do
		if ! rpm -q $RPM &>/dev/null; then
			if [[ $RPM = *.x86_64 ]]; then
				if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
					MISSING="$RPM $MISSING"
				fi
			fi
		fi
	done

	if [ -n "$MISSING" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING ]"${color_off}"
		echo ""
		if [ $OSDIST -eq 0 ]; then
			redhat
		else
			CentOS
		fi
	fi
}

GUI()
{
	print_head

	echo -e "${color_blue}[ Install GUI rpm ]${color_off}"
	echo ""

	unset MISSING
	unset EXIST

	Agent=0
	for RPM_e in ${List_GUI}; do
		if rpm -q $RPM_e &>/dev/null; then
			if [[ $RPM_e = *.x86_64 ]]; then
					EXIST="$RPM_e $EXIST"
			fi
		fi
	done

	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for RPM in ${List_GUI}; do
		if ! rpm -q $RPM &>/dev/null; then
			if [[ $RPM = *.x86_64 ]]; then
				if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
					MISSING="$RPM $MISSING"
				fi
			fi
		fi
	done

	if [ -n "$MISSING" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING ]"${color_off}"
		echo ""
		if [ $OSDIST -eq 0 ]; then
			redhat
		else
			CentOS
		fi
	fi
}

All()
{
	print_head
	echo -e "${color_blue}[ Install All rpm ]${color_off}"
	echo ""

	unset MISSING
	unset EXIST

	Agent=0
	for RPM_e in ${List_All}; do
		if rpm -q $RPM_e &>/dev/null; then
			if [[ $RPM_e = *.x86_64 ]]; then
					EXIST="$RPM_e $EXIST"
			fi
		fi
	done

	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST ]"${color_off}"
	echo ""

	for RPM in ${List_All}; do
		if ! rpm -q $RPM &>/dev/null; then
			if [[ $RPM = *.x86_64 ]]; then
				if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
					MISSING="$RPM $MISSING"
				fi
			fi
		fi
	done

	if [ -n "$MISSING" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING ]"${color_off}"
		echo ""
		if [ $OSDIST -eq 0 ]; then
			redhat
		else
			CentOS
		fi
	fi
}

##################################################
pre_kernel-devel(){

print_head
OS_Ver
Check_OS_Version

List1="kernel-devel-$(uname -r)"
List2="gcc.x86_64 make.x86_64 perl.x86_64"
if [ ${RHELVER} -ge 8 ]; then
	List_datto="epel-release dkms dkms-dattobd libdattobd dattobd-utils"
else
	List_datto="epel-release dkms dkms-dattobd libdattobd dattobd-utils datto-el-rpm-release"
fi
os_ver=`cat /etc/redhat-release | grep -o '[0-9]\.[0-9]*'`
plat=`uname -i`
ker_num=`uname -r | sed -e 's/\(^[0-9]\+\.[0-9]\+\.[0-9]\+\-[0-9]\+\).*$/\1/'`

if [ -f $dir/kernel-release.txt ]; then
	if [ ${RHELVER} -ge 7 ]; then
		release_ver1=`grep "${ker_num}" $dir/kernel-release.txt | awk -F" " '{print $1 }'`
		release_ver=`grep "${ker_num}" $dir/kernel-release.txt | awk -F" " '{print $2 }'`
	fi
	if [ ${RHELVER} -le 6 ]; then
		release_ver=`grep "${ker_num}" $dir/kernel-release.txt | awk -F" " '{print $1 }'`
	fi
else
	echo "kernel-release.txt does not exist."
	echo "Input the kernel-release.txt path [e.g.: /tmp ] : "
	read txt_path
	if [ -f $txt_path/kernel-release.txt ]; then 
		if [ ${RHELVER} -ge 7 ]; then
			release_ver1=`grep "${ker_num}" $txt_path/kernel-release.txt | awk -F" " '{print $1 }'`
			release_ver=`grep "${ker_num}" $txt_path/kernel-release.txt | awk -F" " '{print $2 }'`
			echo $release_ver1
			echo $release_ver
		fi
		if [ ${RHELVER} -le 6 ]; then
			release_ver=`grep "${ker_num}" $txt_path/kernel-release.txt | awk -F" " '{print $1 }'`
		fi
	else
		echo "kernel-release.txt does not exist."
		exit 1
	fi
fi

unset MISSING1
unset MISSING2
unset EXIST1
unset MISSING_datto
unset EXIST_datto

for RPM_e in $List1; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST1="$RPM_e $EXIST1"
		fi
	fi
done

for RPM_e in $List2; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST2="$RPM_e $EXIST2"
		fi
	fi
done

for RPM_e in $List_datto; do
	if rpm -q $RPM_e &>/dev/null; then
		if [[ $RPM_e = *.x86_64 || *.i686 ]]; then
				EXIST_datto="$RPM_e $EXIST_datto"
		fi
	fi
done

for RPM in $List1; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING1="$RPM $MISSING1"
			fi
		fi
	fi
done

for RPM in $List2; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING2="$RPM $MISSING2"
			fi
		fi
	fi
done

for RPM in $List_datto; do
	if ! rpm -q $RPM &>/dev/null; then
		if [[ $RPM = *.x86_64 || *.i686 ]]; then
			if ! rpm -q ${RPM/%.x86_64/.i686} &>/dev/null; then
				MISSING_datto="$RPM $MISSING_datto"
			fi
		fi
	fi
done

}

kernel_devel_install() {

echo -e "${color_blue}[ Install kernel-devel ]${color_off}"

if [ -n "$MISSING1" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 ]"${color_off}"
	echo ""

	vault_remove
	DVD_remove

	if [ $net_ok -eq 0 ]; then 	#つながる場合
		echo "=== Internet connection OK."
		echo "yum clean all"
		yum clean all 
		oracle_repo
		ins_check_devel=`rpm -qa kernel-devel | wc -l`
		if [ $ins_check_devel = 0 ]; then
			if [ "${os_ver}" != "${release_ver}"  ]; then		
				ker_dev_not_inst_not_match
			fi
			if [ "${os_ver}" = "${release_ver}"  ]; then
				ker_dev_not_inst_match
			fi
		else
			if [ "${os_ver}" != "${release_ver}"  ]; then
				ker_dev_inst_not_match
			fi
			if [ "${os_ver}" = "${release_ver}"  ]; then
				ker_dev_inst_match
			fi
		fi
	else		##########################yum がつながらない　DVD参照
		ins_check_devel=`rpm -qa kernel-devel | wc -l`
		if [ $ins_check_devel = 0 ]; then
			if [ "${os_ver}" = "${release_ver}"  ]; then
				echo -e "${color_blue}kernel-devel is not installed.${color_off}"
				echo -e "${color_blue}OS Ver and kernel ver is match.${color_off}"
				uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
				if [ $? -eq 1 ]; then
					echo "base"
					vault_repo_base
				else
					echo -e "${color_red}kernel did minor version up${color_off}"
					echo -e "${color_red}kernel-devel does not install by DVD${color_off}"
					echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
					echo -e "${color_red}Please install "$MISSING1".${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					exit 1
				fi
				read -p "=== Internet connection failed. Please insert the OS DVD."
				echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
				read dvd_path
				dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
				if [ ${RHELVER} = 5 ]; then
					if [ `grep -c "Enterprise" /etc/redhat-release` = 1 ]; then
						$(repo_ent5)
					else			
						$(repo)
					fi
				elif [ ${RHELVER} = 6 -o ${RHELVER} = 7 ]; then
					$(repo)
				elif [ ${RHELVER} = 8 ]; then
					$(repo8)
				fi
				echo ""
				echo -e "${color_blue}Created yum repository. Press any key to continue...${color_off}"	
				echo ""
				if [ ${RHELVER} -ge 5 ]; then
					echo ""
					echo -e "${color_blue}Install the required module using yum command.${color_off}"	
					echo "kernel-devel install"
					if [ ${RHELVER} -eq 8 ]; then
						yum -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $List1
					else
						yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install $List1
					fi
					if [ $? = 1 ]; then
						echo -e "${color_red}yum install is failed.${color_off}"
						echo -e "${color_red}Please get $List1 in another way.${color_off}"
						echo -e "${color_red}Please install $List1.${color_off}"
						echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
						DVD_remove							
						exit 1
					fi
				fi
			else
				echo -e "${color_blue}kernel-devel is not installed.${color_off}"
				echo -e "${color_blue}OS Ver and kernel ver is not match.${color_off}"
				uname -r | grep -oE '\-[0-9]+.[0-9]+.[0-9]+'
				if [ $? -eq 1 ]; then
					echo "base"
					vault_repo_base
				else
					echo -e "${color_red}kernel did minor version up${color_off}"
					echo -e "${color_red}kernel-devel does not install by DVD${color_off}"
					echo -e "${color_red}Please get "$MISSING1" in another way.${color_off}"
					echo -e "${color_red}Please install "$MISSING1".${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					exit 1
				fi
				echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
				read dvd_path
				dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
				if [ ${RHELVER} = 5 ]; then
					if [ `grep -c "Enterprise" /etc/redhat-release` = 1 ]; then
						$(repo_ent5)
					else			
						$(repo)
					fi
				elif [ ${RHELVER} = 6 -o ${RHELVER} = 7 ]; then
					$(repo)
				elif [ ${RHELVER} = 8 ]; then
					$(repo8)
				fi
				echo ""
				echo ""
				if [ ${RHELVER} -ge 5 ]; then
					echo ""
					echo -e "${color_blue}Install the required module using yum command.${color_off}"	
					echo "Install kernel-devel"
					if [ ${RHELVER} -eq 8 ]; then
						yum -y --disablerepo=* --enablerepo=DVD --enablerepo=DVD1 --nogpgcheck install $List1
					else
						yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install $List1
					fi
					if [ $? = 1 ]; then
						echo -e "${color_red}yum install is failed.${color_off}"
						echo -e "${color_red}Please get $List1 in another way.${color_off}"
						echo -e "${color_red}Please install $List1.${color_off}"
						echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
						DVD_remove
						exit 1
					fi
				fi
			fi
		fi
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST1 ]"${color_off}"
	echo ""
fi
DVD_remove
}
##################gcc make section#######################
req_rpm() {

echo -e "${color_blue}[ required rpm packages ]${color_off}"

if [ -n "$MISSING2" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING2 ]"${color_off}"
	echo ""

	vault_remove
	DVD_remove

	echo "yum clean all"
	echo ""
	yum clean all
	echo ""
	echo "yum repolist all ....." 
####yum がつながらない　DVD参照
	repolist=`yum repolist all | grep -o "repolist: 0"`
	if [ $? -eq 0 ]; then
		read -p "=== Yum repository does not exist. Please insert the OS DVD. ==="
		echo -n "Input DVD mount path [e.g.: /run/media/root/CentOS 7 x86_64] :"
		read dvd_path
		dvd_path=`echo "$dvd_path" | sed -e 's/ /%20/g'`
		if [ ${RHELVER} = 5 ]; then
			if [ `grep -c "Enterprise" /etc/redhat-release` = 1 ]; then
				$(repo_ent5)
			else			
				$(repo)
			fi
		else
			$(repo)
		fi
		echo ""
		echo ""
		if [ -n "$MISSING2" ]; then
			echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
			echo ""
			echo -e "${color_magenta}"[ $MISSING2 ]"${color_off}"
			echo ""
			echo yum install "$MISSING2".
			yum -y --disablerepo=* --enablerepo=DVD --nogpgcheck install $MISSING2
				if [ $? -eq 1 ]; then
					echo -e "${color_red}yum install is failed.${color_off}"
					echo -e "${color_red}Please "$MISSING2" in another way.${color_off}"
					echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
					DVD_remove
					exit 1
				fi
		else
			echo ""
			echo -e "${color_blue}Already installed rpm packages:${color_off}"
			echo ""
			echo -e "${color_magenta}"[ $EXIST2 ]"${color_off}"
			echo ""
		fi
	else
		echo "yum repository is retrieved."
		if [ -n "$MISSING2" ]; then
			echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
			echo ""
			echo -e "${color_magenta}"[ $MISSING2 ]"${color_off}"
			echo ""
			echo yum install "$MISSING2".
			yum -y install $MISSING2
			if [ $? -eq 1 ]; then
				echo -e "${color_red}yum install is failed.${color_off}"
				echo -e "${color_red}Please "$MISSING2" in another way.${color_off}"
				echo -e "${color_red}Please check https://pkgs.org/.${color_off}"
				exit 1
			fi
		else
			echo ""
			echo -e "${color_blue}Already installed rpm packages:${color_off}"
			echo ""
			echo -e "${color_magenta}"[ $EXIST2 ]"${color_off}"
			echo ""
		fi
	fi
else
	echo ""
	echo -e "${color_blue}Already installed rpm packages:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST2 ]"${color_off}"
	echo ""
fi
DVD_remove
}

#################datto section######################
datto_rpm () {

	if [ ${RHELVER} -eq 5 ]; then
		pkg_epel=`ls $dir/packages | grep epel-release-5-`
		echo "[ rpm -Uvh $dir/packages/$pkg_epel ]"
		rpm -Uvh $dir/packages/$pkg_epel

		pkg_dkms=`ls $dir/packages | grep dkms-2.2`
		echo "[ rpm -Uvh $dir/packages/$pkg_dkms ]"
		rpm -Uvh $dir/packages/$pkg_dkms

		pkg_datto=`ls $dir/packages/datto | grep datto-el-rpm-release-5`
		echo "[ rpm -Uvh $dir/packages/datto/$pkg_datto ]"
		rpm -Uvh $dir/packages/datto/$pkg_datto

		pkg_dkms_dattobd=`ls $dir/packages/datto | grep dkms-dattobd | grep el5 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd

		pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep el5 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd

		pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep el5 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils
	fi

	if [ ${RHELVER} -eq 6 ]; then
		pkg_epel=`ls $dir/packages | grep epel-release-6-`
		echo "[ rpm -Uvh $dir/packages/$pkg_epel ]"
		rpm -Uvh $dir/packages/$pkg_epel

		pkg_dkms=`ls $dir/packages | grep dkms-2.4.0`
		echo "[ rpm -Uvh $dir/packages/$pkg_dkms ]"
		rpm -Uvh --nodeps $dir/packages/$pkg_dkms

		pkg_datto=`ls $dir/packages/datto | grep datto-el-rpm-release-6`
		echo "[ rpm -Uvh $dir/packages/datto/$pkg_datto ]"
		rpm -Uvh $dir/packages/datto/$pkg_datto

		pkg_dkms_dattobd=`ls $dir/packages/datto | grep dkms-dattobd | grep el6 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd

		pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep el6 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd

		pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep el6 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils

	fi

	if [ ${RHELVER} -eq 7 ]; then
		pkg_epel=`ls $dir/packages | grep epel-release-7-`
		echo "[ rpm -Uvh $dir/packages/$pkg_epel ]"
		rpm -Uvh $dir/packages/$pkg_epel

		pkg_dkms=`ls $dir/packages | grep dkms-2.7.1 | grep el7 | head -1`
		echo "[ rpm -Uvh $dir/packages/$pkg_dkms ]"		
		rpm -Uvh --nodeps $dir/packages/$pkg_dkms

		pkg_datto=`ls $dir/packages/datto | grep datto-el-rpm-release-7`
		echo "[ rpm -Uvh $dir/packages/datto/$pkg_datto ]"
		rpm -Uvh $dir/packages/datto/$pkg_datto

		pkg_dkms_dattobd=`ls $dir/packages/datto | grep dkms-dattobd | grep el7 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd

		pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep el7 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd

		pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep el7 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils
	fi

	if [ ${RHELVER} -eq 8 ]; then
		pkg_epel=`ls $dir/packages | grep epel-release-8-`
		echo "[ rpm -Uvh $dir/packages/$pkg_epel ]"
		rpm -Uvh $dir/packages/$pkg_epel

		pkg_dkms=`ls $dir/packages | grep dkms-2.7.1 | grep el8 | head -1`
		echo "[ rpm -Uvh $dir/packages/$pkg_dkms ]"		
		rpm -Uvh --nodeps $dir/packages/$pkg_dkms

		#pkg_datto=`ls $dir/packages/datto | grep datto-el-rpm-release-8`
		#echo "[ rpm -Uvh $dir/packages/datto/$pkg_datto ]"
		#rpm -Uvh $dir/packages/datto/$pkg_datto

		pkg_dkms_dattobd=`ls $dir/packages/datto | grep dkms-dattobd | grep el8 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_dkms_dattobd

		pkg_libdattobd=`ls $dir/packages/datto | grep libdattobd | grep el8 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_libdattobd

		pkg_datto_utils=`ls $dir/packages/datto | grep dattobd-utils | grep el8 | head -1`
		echo "[ rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils ]"
		rpm -Uvh --force $dir/packages/datto/$pkg_datto_utils
	fi
}

req_datto_rpm() {

if [ -n "$MISSING1" -o -n "$MISSING2" ]; then
	echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $MISSING1 $MISSING2 ]"${color_off}"
	echo ""
	echo -e "${color_red}There are no required packages. ${color_off}"
	if [ -n "$MISSING1" -a -n "$MISSING2" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] and [4] first. ${color_off}"	
	elif [ -n "$MISSING2" ]; then
		echo -e "${color_red}Please run the [[1] or [3]] first. ${color_off}"
	elif [ -n "$MISSING1" ]; then
		echo -e "${color_red}Please run the [4] first. ${color_off}"
	fi
	req_datto=1
else
	req_datto=0
fi

}

datto_install() {
print_head
OS_Ver
Check_OS_Version

echo -e "${color_blue}[ datto driver install ]${color_off}"
req_datto_rpm
if [ ${req_datto} -eq 0 ]; then
	if [ -n "$MISSING_datto" ]; then
		echo -e "${color_blue}The following rpm packages are not installed:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $MISSING_datto ]"${color_off}"
		echo ""

		if [ -e $dir/packages -a -e $dir/packages/datto ]; then
			datto_rpm
		else
			echo "datto package is not exist."
			echo -n "Input packages path [e.g.: /tmp ] :"
			read dir
			datto_rpm
		fi

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}dattobd Driver install successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver install failed.${color_off}"
		fi
		
		insmod /lib/modules/`uname -r`/extra/dattobd.ko > /dev/null 2>&1 || true

		echo ""
		echo "[ /proc/datto-info ]"
		cat /proc/datto-info

		dbdctl help > /dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo -e "${color_blue}datto utils install successful.${color_off}" 
		else
			echo -e "${color_red}datto utils install failed.${color_off}"	
		fi

	else
		echo ""
		echo -e "${color_blue}Already installed rpm packages:${color_off}"
		echo ""
		echo -e "${color_magenta}"[ $EXIST_datto ]"${color_off}"
		echo ""
	fi
fi
}

ask_YN ()
{
	local ans="invalid"
	local default=Y
	local defstr="[Y/n]"

	if [[ "$2" = [nN] ]]; then
		default="N"
		defstr="[y/N]"
	fi

	echo -n "$1 $defstr "
	read -n1 ans
	while [[ $ans != *[yYnN] ]]; do
		echo
		echo -n "$1 $defstr "
		read -n1 ans
	done
	[ "$ans" = "" ] && ans=$default
	echo

	RC_ask_YN="Y"
	[[ $ans = [nN] ]] && RC_ask_YN="N"
}

datto_rpm_remove() {

	ask_YN "Do you want to uninstall datto driver and rpm?" "Y"
	if [ "$RC_ask_YN" = "Y" ]; then
		for rpm in $EXIST_datto; do
			echo "[ rpm -e $rpm ]"
			rpm -e $rpm
		done
		#yum -y remove $EXIST_datto

		modinfo dattobd > /dev/null 2>&1
		if [ $? -eq 1 ]; then
			echo -e "${color_blue}dattobd Driver uninstall successful.${color_off}"	
		else
			echo -e "${color_red}dattobd Driver uninstall failed.${color_off}"
		fi

		dbdctl help > /dev/null 2>&1
		if [ $? -eq 127 ]; then
			echo -e "${color_blue}datto utils uninstall successful.${color_off}" 
		else
			echo -e "${color_red}datto utils uninstall failed.${color_off}"	
		fi
	fi
}

datto_remove() {
print_head
OS_Ver
Check_OS_Version

echo -e "${color_blue}[ datto driver uninstall ]${color_off}"
echo ""

if [ -n "$EXIST_datto" ]; then
	echo -e "${color_blue}The following rpm packages are installed:${color_off}"
	echo ""
	echo -e "${color_magenta}"[ $EXIST_datto ]"${color_off}"
	echo ""
	datto_rpm_remove
else
	echo -e "${color_blue}The datto rpm packages are not exists.${color_off}"
	echo ""
fi

}

#######################################################################
# Main
#######################################################################

show_help()
{
	print_head

	echo "Options:"
	printf "\tAgent\t\t\t: Install required packages for Agent.\n"
	printf "\tGUI\t\t\t: Install required packages for GUI.\n"
	printf "\t[1]-[2]All\t\t: Install required packages for Agent and GUI.\n"
	printf "\tKernel-devel\t\t: Install required Kernel-devel.\n"
	printf "\tInstall datto driver\t: Install datto driver.\n"
	printf "\tUninstall datto driver\t: Uninstall datto driver.\n"
	printf "\thelp\t\t\t: Print this help.\n"
	printf "\tExit\t\t\t: Exit.\n"
	echo ""
}

OS_Ver
Check_OS_Version
Check_sb

case "$RHELVER" in
5)
	List_Agent="make.x86_64 gcc.x86_64 tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 e2fsprogs-libs.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64"
	;;
6)
	List_Agent="make.x86_64 gcc.x86_64 tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64"
	
	List_GUI="tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64"

	List_All="make.x86_64 gcc.x86_64 tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64"
	;;
7)
	List_Agent="make.x86_64 gcc.x86_64 tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64"

	List_GUI="tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64"

	List_All="make.x86_64 gcc.x86_64 tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64"
	;;

8)
	List_Agent="make.x86_64 gcc.x86_64 perl.x86_64 tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 elfutils-libelf-devel.x86_64 libnsl.x86_64"

	List_GUI="tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64  zlib-devel.x86_64"

	List_All="make.x86_64 gcc.x86_64 perl.x86_64 tar.x86_64 gzip.x86_64 unzip.x86_64 vim-common.x86_64 wget.x86_64 fuse.x86_64 glibc.x86_64 libgcc.x86_64 libstdc++.x86_64 libuuid.x86_64 nss-softokn-freebl.x86_64 zlib.x86_64 expat.x86_64 fontconfig.x86_64 freetype.x86_64 glib2.x86_64 libICE.x86_64 libSM.x86_64 libX11.x86_64 libXau.x86_64 libXext.x86_64 libXrender.x86_64 libxcb.x86_64 mesa-libGL.x86_64 systemd-libs.x86_64 elfutils-libelf-devel.x86_64 zlib-devel.x86_64 libnsl.x86_64"
	;;
esac

if [ -f /etc/yum.repos.d/DVD.repo ]; then
	rm /etc/yum.repos.d/DVD.repo
fi

Check_Dist
Check_internet
Check_subsc

if [ "$RHELVER" == "7" ] ; then
	V2=`cat /etc/redhat-release | grep -oE '[0-9]+\.[0-9]+'`
	result=`echo "$V2 > 7.5" | bc`
	if [ "$net_ok" == "0" ]; then
		if [ $result -ne 1 ] ; then
			echo -e "${color_red}This OS version [$V2], it has been confirmed that installing a package over the insternet causes [Oh no!] symptom.${color_off}"
			echo -e "${color_red}yum command has been aborted over the Internet.${color_off}"
			echo -e "${color_blue}Please try Ver7.5 or later.${color_off}"
			echo -e "${color_blue}Or please run the script without internet connection.${color_off}"
			exit
		fi
	fi
fi

if [ $RHELVER -eq 5 ]; then
	select_options_5
elif [ $RHELVER -eq 8 ]; then
	select_options
else
	select_options
fi

vault_remove
DVD_remove
ol_remove
echo "** `date '+%Y-%m-%d %H:%M:%S'` -END"
exit 0
